@@ -9,7 +9,9 @@ use crate::alloc::{vec, vec::Vec};
 use core::any::TypeId;
 use core::borrow::Borrow;
 use core::convert::TryFrom;
-use core::sync::atomic::{AtomicU64, Ordering};
+use core::marker::PhantomData;
+use spin::Mutex;
+
 use core::{fmt, mem, ptr};
 
 #[cfg(feature = "std")]
@@ -21,8 +23,8 @@ use crate::alloc::boxed::Box;
 use crate::archetype::{Archetype, TypeIdMap, TypeInfo};
 use crate::entities::{Entities, EntityMeta, Location, ReserveEntitiesIterator};
 use crate::{
-    Bundle, ColumnBatch, DynamicBundle, Entity, EntityRef, Fetch, MissingComponent, NoSuchEntity,
-    Query, QueryBorrow, QueryItem, QueryMut, QueryOne, Ref, RefMut,
+    Bundle, Column, ColumnBatch, ColumnMut, DynamicBundle, Entity, EntityRef, Fetch,
+    MissingComponent, NoSuchEntity, Query, QueryBorrow, QueryItem, QueryMut, QueryOne, Ref, RefMut,
 };
 
 /// An unordered collection of entities, each having any number of distinctly typed components
@@ -54,15 +56,20 @@ pub struct World {
 impl World {
     /// Create an empty world
     pub fn new() -> Self {
-        static ID: AtomicU64 = AtomicU64::new(1);
-
+        // AtomicU64 is unsupported on 32-bit MIPS and PPC architectures
+        // For compatibility, use Mutex<u64>
+        static ID: Mutex<u64> = Mutex::new(1);
+        let id = {
+            let mut id = ID.lock();
+            let next = id.checked_add(1).unwrap();
+            *id = next;
+            next
+        };
         Self {
             entities: Entities::default(),
             archetypes: ArchetypeSet::new(),
             bundle_to_archetype: HashMap::default(),
-            id: ID
-                .fetch_update(Ordering::Relaxed, Ordering::Relaxed, |id| id.checked_add(1))
-                .unwrap(),
+            id,
         }
     }
 
@@ -339,10 +346,10 @@ impl World {
     /// Prefer [`query_mut`](Self::query_mut) when concurrent access to the [`World`] is not required.
     ///
     /// Calling `iter` on the returned value yields `(Entity, Q)` tuples, where `Q` is some query
-    /// type. A query type is `&T`, `&mut T`, a tuple of query types, or an `Option` wrapping a
-    /// query type, where `T` is any component type. Components queried with `&mut` must only appear
-    /// once. Entities which do not have a component type referenced outside of an `Option` will be
-    /// skipped.
+    /// type. A query type is any type for which an implementation of [`Query`] exists, e.g. `&T`,
+    /// `&mut T`, a tuple of query types, or an `Option` wrapping a query type, where `T` is any
+    /// component type. Components queried with `&mut` must only appear once. Entities which do not
+    /// have a component type referenced outside of an `Option` will be skipped.
     ///
     /// Entities are yielded in arbitrary order.
     ///
@@ -768,6 +775,37 @@ impl World {
         self.archetypes_inner().iter()
     }
 
+    /// Borrow every `T` component for efficient random access
+    ///
+    /// [`Column::get`] is semantically equivalent to [`World::get`], except that every `T`
+    /// component is borrowed in advance, and repeated calls are much cheaper, at the cost of
+    /// additional work when the [`Column`] is fetched.
+    ///
+    /// Panics if a unique borrow is outstanding for any `T` component.
+    ///
+    /// # Example
+    /// ```
+    /// use::hecs::*;
+    /// let mut world = World::new();
+    /// let ent = world.spawn((123, "abc"));
+    /// let column = world.column::<i32>();
+    /// assert_eq!(*column.get(ent).unwrap(), 123);
+    /// ```
+    pub fn column<T: Component>(&self) -> Column<'_, T> {
+        let archetypes = self.archetypes.archetypes.as_slice();
+        let entities = self.entities.meta.as_slice();
+        Column::new(entities, archetypes, PhantomData)
+    }
+
+    /// Uniquely borrows every `T` component for efficient random access
+    ///
+    /// See [`World::column`].
+    pub fn column_mut<T: Component>(&self) -> ColumnMut<'_, T> {
+        let archetypes = self.archetypes.archetypes.as_slice();
+        let entities = self.entities.meta.as_slice();
+        ColumnMut::new(entities, archetypes, PhantomData)
+    }
+
     /// Returns a distinct value after `archetypes` is changed
     ///
     /// Store the current value after deriving information from [`archetypes`](Self::archetypes),
