@@ -272,6 +272,29 @@ fn dynamic_components() {
     );
 }
 
+#[test]
+fn spawn_buffered_entity() {
+    let mut world = World::new();
+    let mut buffer = CommandBuffer::new();
+    let ent = world.reserve_entity();
+    let ent1 = world.reserve_entity();
+    let ent2 = world.reserve_entity();
+    let ent3 = world.reserve_entity();
+
+    buffer.insert(ent, (1, true));
+    buffer.insert(ent1, (13, 7.11, "hecs"));
+    buffer.insert(ent2, (17 as i8, false, 'o'));
+    buffer.insert(ent3, (2 as u8, "qwe", 101.103, false));
+
+    buffer.run_on(&mut world);
+
+    assert_eq!(*world.get::<bool>(ent).unwrap(), true);
+    assert_eq!(*world.get::<&str>(ent1).unwrap(), "hecs");
+    assert_eq!(*world.get::<i32>(ent1).unwrap(), 13);
+    assert_eq!(*world.get::<bool>(ent2).unwrap(), false);
+    assert_eq!(*world.get::<u8>(ent3).unwrap(), 2);
+}
+
 #[test]
 #[should_panic(expected = "already borrowed")]
 fn illegal_borrow() {
@@ -566,3 +589,43 @@ fn empty_entity_ref() {
     let r = world.entity(e).unwrap();
     assert_eq!(r.entity(), e);
 }
+
+#[test]
+fn query_or() {
+    let mut world = World::new();
+    let e = world.spawn(("abc", 123));
+    let _ = world.spawn(("def",));
+    let f = world.spawn(("ghi", true));
+    let g = world.spawn(("jkl", 456, false));
+    let results = world
+        .query::<(&&str, Or<&i32, &bool>)>()
+        .iter()
+        .map(|(handle, (&s, value))| (handle, s, value.cloned()))
+        .collect::<Vec<_>>();
+    assert_eq!(results.len(), 3);
+    assert!(results.contains(&(e, "abc", Or::Left(123))));
+    assert!(results.contains(&(f, "ghi", Or::Right(true))));
+    assert!(results.contains(&(g, "jkl", Or::Both(456, false))));
+}
+
+#[test]
+fn column_get() {
+    let mut world = World::new();
+    let ent = world.spawn((123, "abc"));
+    let ent2 = world.spawn((true, "hecs"));
+    let column = world.column::<&str>();
+    assert_eq!(*column.get(ent).unwrap(), "abc");
+    assert_eq!(*column.get(ent2).unwrap(), "hecs");
+}
+
+#[test]
+fn column_get_mut() {
+    let mut world = World::new();
+    let ent = world.spawn((0, true));
+    {
+        let mut column = world.column_mut::<i32>();
+        *column.get(ent).unwrap() = 99;
+        assert_eq!(*column.get(ent).unwrap(), 99);
+    }
+    assert_eq!(*world.get::<i32>(ent).unwrap(), 99);
+}
